@@ -1,5 +1,9 @@
 /*
  * Copyright (C) 2003  Stefan Kleine Stegemann
+ *           (C) 2013-2016  The GNUstep Application Team
+ *
+ * Authors: Stefan Kleine Stegemann
+ *          Riccardo Mottola
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,19 +20,20 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-#include "PDFDocument.h"
-#include "PDFImageRep.h"
-#include "XPDFBridge.h"
-#include "CountingRef.h"
-#include "PDFOutline.h"
+#import "PDFDocument.h"
+#import "PDFImageRep.h"
+#import "XPDFBridge.h"
+#import "CountingRef.h"
+#import "PDFOutline.h"
 
 #define CONTENT_SIZE 262144
 
-static NSMutableString *textContents = nil;
-static NSMutableDictionary *documentInfo = nil;
 
+@interface PDFDocRefDelegate : NSObject <CountingRefDelegate>
+{
+}
+@end
 
-@class PDFDocRefDelegate;
 
 /*
  * Non-Public methods of PDFDocument.
@@ -66,7 +71,7 @@ static NSMutableDictionary *documentInfo = nil;
 
 - (void) dealloc
 {
-  NSLog(@"dealloc PDFDocument, retain count is %d", [self retainCount]);
+  NSLog(@"dealloc PDFDocument, retain count is %lu", (unsigned long)[self retainCount]);
   RELEASE (outline);
   RELEASE (pdfDocRef);
   RELEASE (textContents);
@@ -139,7 +144,7 @@ static NSMutableDictionary *documentInfo = nil;
 {
    NSAssert(![pdfDocRef isNULL], @"no document");
    
-   return PDFDoc_getPageWidth([pdfDocRef pointer], pageNum);
+   return PDFDoc_getPageCropWidth([pdfDocRef pointer], pageNum);
 }
 
 
@@ -147,7 +152,7 @@ static NSMutableDictionary *documentInfo = nil;
 {
    NSAssert(![pdfDocRef isNULL], @"no document");
    
-   return PDFDoc_getPageHeight([pdfDocRef pointer], pageNum);
+   return PDFDoc_getPageCropHeight([pdfDocRef pointer], pageNum);
 }
 
 
@@ -160,12 +165,12 @@ static NSMutableDictionary *documentInfo = nil;
    {
       if (([self pageRotate: pageNum] == 90) || ([self pageRotate: pageNum] == 180))
       {
-         float height = size.height;
+         CGFloat height = size.height;
          size.height = size.width;
          size.width  = height;
       }
    }
-
+   NSLog(@"PDFDocument pageSize: %f %f", size.width, size.height);
    return size;
 }
 
@@ -180,11 +185,15 @@ static NSMutableDictionary *documentInfo = nil;
 
 - (int) countPages
 {
-   NSAssert(![pdfDocRef isNULL], @"no document");
-   
-   return PDFDoc_getNumPages([pdfDocRef pointer]);
+   return (int) [self pageCount];
 }
 
+- (NSUInteger) pageCount
+{
+   NSAssert(![pdfDocRef isNULL], @"no document");
+   
+   return (NSUInteger) PDFDoc_getNumPages([pdfDocRef pointer]);  
+}
 
 - (NSString*)metaData
 {
@@ -279,6 +288,7 @@ usingSearchContext: (PDFSearchContext*)aSearchContext
                            [aText cString],
                            pageNum,
                            maxPage,
+                           0,
                            &xMin,
                            &yMin,
                            &xMax,
@@ -324,7 +334,12 @@ usingSearchContext: (PDFSearchContext*)aSearchContext
 
 static void outputToString(void *stream, char *text, int len) 
 {
-  [textContents appendString: [NSString stringWithUTF8String: text]];
+  NSMutableString *ms;
+
+  ms = (NSMutableString *)stream;
+  [ms appendString: [[[NSString alloc] initWithBytes:text
+					      length:(NSUInteger)len
+					    encoding:NSUTF8StringEncoding] autorelease]];
 }
 
 - (NSString *)getAllText
@@ -333,7 +348,7 @@ static void outputToString(void *stream, char *text, int len)
   
   [textContents setString: @""];
   
-  if (PDFUtil_GetAllText([pdfDocRef pointer], outputToString) == 0) {
+  if (PDFUtil_GetAllText([pdfDocRef pointer], outputToString, textContents) == 0) {
     DESTROY (textContents);
   } 
 
@@ -342,12 +357,13 @@ static void outputToString(void *stream, char *text, int len)
   return textContents;
 }
 
-static void getDictPair(char *key, char *value) 
+static void getDictPair(char *key, char *value, void *iD) 
 {
   if (key != NULL && strlen(key) && value != NULL && strlen(value)) {
     NSString *keystr = nil;
     NSString *valuestr = nil;
     BOOL pairok = YES;
+    NSMutableDictionary *documentInfo = (NSMutableDictionary *)iD;    
     
     NS_DURING
 	    {
@@ -373,7 +389,7 @@ static void getDictPair(char *key, char *value)
 
   [documentInfo removeAllObjects];
 
-  if (PDFUtil_GetInfo([pdfDocRef pointer], getDictPair) == 0) {
+  if (PDFUtil_GetInfo([pdfDocRef pointer], getDictPair, documentInfo) == 0) {
     DESTROY (documentInfo);
   } 
 
@@ -476,10 +492,7 @@ static void getDictPair(char *key, char *value)
 /*  Class PDFDocRefDelegate                              */
 /* ----------------------------------------------------- */
 
-@interface PDFDocRefDelegate : NSObject <CountingRefDelegate>
-{
-}
-@end
+
 
 
 @implementation PDFDocRefDelegate
